@@ -4,10 +4,10 @@ use strict;
 use warnings;
 use UNIVERSAL::require;
 use YAML::Syck;
-our $VERSION='0.61';
-
+our $VERSION='0.73';
+use Try::Tiny;
 use File::Copy;
-
+#
 sub new{
   my ($class, $r,$prefix)=@_;
   my $self = {};
@@ -195,8 +195,6 @@ sub add_element{
   }
 
   if (($params->{type} eq 'Image')||($params->{type} eq 'Upload')) {
-    
-    
     if ( exists $self->{request}->{$name.'_submit' } ){
       $self->{submitted}=1 ;
       $self->{submit_value} = $namew.'_uploaded';
@@ -231,14 +229,20 @@ sub add_element{
     }
   }
 
-  if ($params->{type} eq 'Captcha') {
+  if ($params->{type} eq 'Captcha') {      
+      my $tlabel=$params->{label1};
+      my $tlabel2=$params->{label2};
       my $tname=$name."_input";
       my $tname2=$name."_input2";
-      my $c_val = $self->get_value($name);
-      $self->add_element({ type => 'Text',  name => $tname } );
-      $self->add_element({ type => 'Text',  name => $tname2, class=>"form_input2" } );
+      $self->add_element({ type => 'Text',  name => $tname, label=> $tlabel } );
+      $self->add_element({ type => 'Text',  name => $tname2, class=>"form_input2", label=> $tlabel2 } );
+      my $c_val = $self->get_value($tname2);
+      
+      #use Data::Dumper;
+      #print STDERR Dumper($params);
       $self->add_constraint({ type=> 'Equation', operator=>'eq', name=>$tname, comp=>$c_val, text=>$params->{message} });
-      $self->add_constraint({ type=> 'Equation', operator=>'eq', name=>$tname2, comp=>'', text=>$params->{message} });
+      #$self->add_constraint({ type=> 'Equation', operator=>'eq', name=>$tname2, comp=>'', text=>$params->{message} });      
+      $self->add_constraint({ type=> 'Mintime', name=>$tname, keyname=> $params->{keyname}."2", keyphrase=>$params->{keyphrase} ,session=> $params->{session} , text=>'Error, please wait 5 Seconds and resubmit the form.' });                      
   }
 }
 
@@ -450,12 +454,25 @@ sub get_dbix{
 
 sub add_options{
   my ($self,$name,$options)=@_;
-  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->add_options($options);
+  $self->{prefix}='' if (!$self->{prefix});
+  if ($name && $name ne ''){
+     if ($self->{element_index}->{$self->{prefix}.$name }){      
+        $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->add_options($options);
+     }
+  }
 }
 
 sub reset_options{
   my ($self,$name,$options,$label,$id)=@_;
-  $self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->reset_options($options,$label,$id);
+  
+  $self->{prefix}='' if (!$self->{prefix});
+  if ($name && $name ne ''){
+     if ($self->{element_index}->{$self->{prefix}.$name }){      
+        my $element=$self->{element}[ $self->{element_index}->{ $self->{prefix}.$name }->{index}
+        ];      
+      $element->reset_options($options,$label,$id) if ($element && $options && $label && $id);
+      }
+  }  
 }
 
 sub freeze{
@@ -471,6 +488,7 @@ sub get_r{
 }
 sub get_e{
   my ($self, $name)=@_;
+  
   return '' if (!$self->{element_index}->{$self->{prefix}.$name}->{error_message});
   return $self->{element_index}->{$self->{prefix}.$name}->{error_message};
 }
@@ -502,13 +520,22 @@ sub unfreeze{
 sub get_value{
   my ($self, $name)=@_;
   my $result='';
-  $result=$self->{element}[$self->{element_index}->{$self->{prefix}.$name}->{index}]->get_value();
+  
+  my $id='';
+   try{
+       $id=$self->{element_index}->{$self->{prefix}.$name}->{index};
+   };
+   
+   $result=$self->{element}[$id]->get_value() if ($id);
+  
   return $result;
 }
 
 sub populate{
   my ($self, $data, $anyway)=@_;
-
+  
+  $self->{submit_value}='' unless ($self->{submit_value});
+  
   if (($self->{submit_value} eq '') or ($anyway ne '')) {
     if (ref($data) eq 'HASH') {
       while (my ($key, $value) = each %{ $data }){
